<?php
/**
 * RoundCube Calendar
 *
 * Some utility functions.
 * - Import from and export to iCalendar (.ics) format.
 * - Convert events from backend to JSON.
 *
 * @version 0.2 BETA 2
 * @author Lazlo Westerhof
 * @author Michael Duelli
 * @url http://rc-calendar.lazlo.me
 * @licence GNU GPL
 * @copyright (c) 2010 Lazlo Westerhof - Netherlands
 */
class Utils
{
  private $rcmail = null;
  private $backend = null;

  public function __construct($rcmail, $backend) {
    $this->rcmail = $rcmail;
    $this->backend = $backend;
  }

  /**
   * Import events from iCalendar format
   *
   * @param  array Associative events array
   * @access public
   */
  public function importEvents($events) {
    //TODO
    // for ($events as $event)
    //   $this->backend->newEvent(...);
  }
  
  /**
   * Export events to iCalendar format
   *
   * @param  integer Start time events window ('Y-m-d H:i:s')
   * @param  integer End time events window ('Y-m-d H:i:s')
   * @return string  Events in iCalendar format (http://tools.ietf.org/html/rfc5545)
   * @access public
   */
  public function exportEvents($start, $end) {
    if (!empty($this->rcmail->user->ID)) {
      $events = $this->backend->getEvents($start, $end);

      $ical = "BEGIN:VCALENDAR\n";
      $ical .= "VERSION:2.0\n";
      $ical .= "PRODID:-//RoundCube Webmail//NONSGML Calendar//EN\n";
      foreach ($events as $event) {
        $ical .= "BEGIN:VEVENT\n";
        $ical .= "DTSTART:" . date('Ymd\THis\Z',$event['start'] - date('Z')) . "\n";
        if($start != $end) {
          $ical .= "DTEND:" . date('Ymd\THis\Z',$event['end'] - date('Z')) . "\n";
        }
        $ical .= "SUMMARY:" . $event['summary'] . "\n";
        $ical .= "DESCRIPTION:" . $event['description'] . "\n";
        if(!empty($event['location'])) {
          $ical .= "LOCATION:" . $event['location'] . "\n";
        }
        if(!empty($event['categories'])) {
          $ical .= "CATEGORIES:" . strtoupper($event['categories']) . "\n";
        }
        $ical .= "END:VEVENT\n";
      }
      $ical .= "END:VCALENDAR";

      return $ical;
    }
  }

  /**
   * Get events from database as JSON
   *
   * @param  integer Start time events window ('Y-m-d H:i:s')
   * @param  integer End time events window ('Y-m-d H:i:s')
   * @return string  JSON encoded events
   * @access public
   */
  public function jsonEvents($start, $end) {
    $events = $this->backend->getEvents($start, $end);
    
    $json = array();
    foreach ($events as $event) {
      $json[]=array( 
        'id'    => $event['event_id'],
        'start' => date('c', $event['start']), // ISO 8601 date (added in PHP 5)
        'end'   => date('c', $event['end']), // ISO 8601 date (added in PHP 5)
        'title' => $event['summary'],
        'description'  => $this->ascii2br($event['description']),
        'location'    => $event['location'],
        'className'  => $event['categories'],
        'allDay'=> ($event['all_day'] == 1)?true:false,
      );
    }
    return json_encode($json);
  }

  /**
   * Get events from database as an associative array
   *
   * @param  integer Start time events window ('Y-m-d H:i:s')
   * @param  integer End time events window ('Y-m-d H:i:s')
   * @return string  Associative events array
   * @access public
   */
  public function arrayEvents($start, $end) {
    $events = $this->backend->getEvents($start, $end);

    return $events;
  }

  public function ascii2br($string) {
  	$ascii = "";
  	
  	$parts = explode("\n", $string);
  	for($i = 0; $i < count($parts); $i++) {
  		$part = $parts[$i];
  		if (empty($part))
  			$ascii .= '<br />';
  		else if ($i + 1 < count($parts))
  			$ascii .= $part . '<br />';
  		else
  			$ascii .= $part;
  	}
  	
  	return $ascii;
  }

  /**
   * Is this an all-day event
   * @param start A GMT time stamp ('Y-m-d H:i:s')
   * @param end A GMT time stamp ('Y-m-d H:i:s')
   * @return bool true for all-day event false otherwise
   */
  private function isAllDay($start, $end) {
  	$allDay = 0;

	$localStart = $this->fromGMT($start);
	$localEnd = $this->fromGMT($end);

  	// Is start and end = midnight
  	if (date('His',$localStart) != '000000' ||
  			date('His',$localEnd) != '000000')
  		return $allDay;
  	// Is diff exactly 1 day
  	$allDay = $localEnd - $localStart;

  	return ($allDay == 0) ? 1 : 0;
  }

  private function fromGMT($datetime) {
    if ($this->rcmail->config->get('timezone') === "auto") {
      $tz = isset($_SESSION['timezone']) ? $_SESSION['timezone'] : date('Z')/3600;
    } else {
      $tz = $this->rcmail->config->get('timezone');
      if($this->rcmail->config->get('dst_active')) {
        $tz++;
      }
    }
    // compensate for servers timezone
    $tz -= date('Z')/3600;

    $timestamp = $datetime + ($tz * 3600);

    return $timestamp;
  }

  /**
   * Get start and end for an all-day event
   * @param start A GMT time stamp ('Y-m-d H:i:s')
   * @param end A GMT time stamp ('Y-m-d H:i:s')
   * @return stdClass (obj->start and obj->end)
   */
  private function getAllDay($start, $end) {
	// all day apparently delivered as UTC
	$res = new stdClass;

   	$res->start = $this->GMT_to_local($start);

	$tmp = date("Ymd", strtotime($end) + (60*60*24));
	$res->end = $this->GMT_to_local($tmp);

	return $res;
  }

  /**
   * Convert a GMT time to local
   * @param datetime A GMT time stamp ('Y-m-d H:i:s')
   * @return iCal datetime
   */
  private function GMT_to_local($datetime) {
	$lt = localtime(strtotime($datetime));
	$local = ($lt[5]+1900);
	$local .= (($lt[4]+1) < 10) ? "0".($lt[4]+1) : ($lt[4]+1);
	$local .= (($lt[3]+1) < 10) ? "0".($lt[3]+1) : ($lt[3]+1);

	return $local;
  }

}
?>
